import os
from clang.cindex import Config
if 'CLANG_LIBRARY_PATH' in os.environ:
    Config.set_library_path(os.environ['CLANG_LIBRARY_PATH'])

from clang.cindex import *
import os
import unittest


kInputsDir = os.path.join(os.path.dirname(__file__), 'INPUTS')


class TestIndex(unittest.TestCase):
    def test_create(self):
        index = Index.create()

    # FIXME: test Index.read

    def test_parse(self):
        index = Index.create()
        self.assertIsInstance(index, Index)
        tu = index.parse(os.path.join(kInputsDir, 'hello.cpp'))
        self.assertIsInstance(tu, TranslationUnit)
        tu = index.parse(None, ['-c', os.path.join(kInputsDir, 'hello.cpp')])
        self.assertIsInstance(tu, TranslationUnit)



import os
from clang.cindex import Config
if 'CLANG_LIBRARY_PATH' in os.environ:
    Config.set_library_path(os.environ['CLANG_LIBRARY_PATH'])

from clang.cindex import CursorKind
from clang.cindex import Index
from clang.cindex import SourceLocation
from clang.cindex import SourceRange
from clang.cindex import TokenKind

from .util import get_tu

import unittest


class TestTokens(unittest.TestCase):
    def test_token_to_cursor(self):
        """Ensure we can obtain a Cursor from a Token instance."""
        tu = get_tu('int i = 5;')
        r = tu.get_extent('t.c', (0, 9))
        tokens = list(tu.get_tokens(extent=r))

        self.assertEqual(len(tokens), 4)
        self.assertEqual(tokens[1].spelling, 'i')
        self.assertEqual(tokens[1].kind, TokenKind.IDENTIFIER)

        cursor = tokens[1].cursor
        self.assertEqual(cursor.kind, CursorKind.VAR_DECL)
        self.assertEqual(tokens[1].cursor, tokens[2].cursor)

    def test_token_location(self):
        """Ensure Token.location works."""

        tu = get_tu('int foo = 10;')
        r = tu.get_extent('t.c', (0, 11))

        tokens = list(tu.get_tokens(extent=r))
        self.assertEqual(len(tokens), 4)

        loc = tokens[1].location
        self.assertIsInstance(loc, SourceLocation)
        self.assertEqual(loc.line, 1)
        self.assertEqual(loc.column, 5)
        self.assertEqual(loc.offset, 4)

    def test_token_extent(self):
        """Ensure Token.extent works."""
        tu = get_tu('int foo = 10;')
        r = tu.get_extent('t.c', (0, 11))

        tokens = list(tu.get_tokens(extent=r))
        self.assertEqual(len(tokens), 4)

        extent = tokens[1].extent
        self.assertIsInstance(extent, SourceRange)

        self.assertEqual(extent.start.offset, 4)
        self.assertEqual(extent.end.offset, 7)



import os
from clang.cindex import Config
if 'CLANG_LIBRARY_PATH' in os.environ:
    Config.set_library_path(os.environ['CLANG_LIBRARY_PATH'])

from clang.cindex import LinkageKind
from clang.cindex import Cursor
from clang.cindex import TranslationUnit

from .util import get_cursor
from .util import get_tu

import unittest


class TestLinkage(unittest.TestCase):
    def test_linkage(self):
        """Ensure that linkage specifers are available on cursors"""

        tu = get_tu("""
void foo() { int no_linkage; }
static int internal;
namespace { struct unique_external_type {} }
unique_external_type unique_external;
extern int external;
""", lang = 'cpp')

        no_linkage = get_cursor(tu.cursor, 'no_linkage')
        self.assertEqual(no_linkage.linkage, LinkageKind.NO_LINKAGE)

        internal = get_cursor(tu.cursor, 'internal')
        self.assertEqual(internal.linkage, LinkageKind.INTERNAL)

        unique_external = get_cursor(tu.cursor, 'unique_external')
        self.assertEqual(unique_external.linkage, LinkageKind.UNIQUE_EXTERNAL)

        external = get_cursor(tu.cursor, 'external')
        self.assertEqual(external.linkage, LinkageKind.EXTERNAL)
